@@ -41,26 +41,32 @@
  *   Bug in verifying the -s (spare) parameter
  * V0.9.6  2013-04-09
  *   Added man page
+ * V0.9.7  2018-03-29
+ *   Directory creation doesn't fail, if it already exists
+ *   Fix compiler warnings for newer GCC
  */
 
-#define VERSION		"0.9.6"
+#define VERSION		"0.9.7"
 
 /* check if lutimes is available */
 #if defined(__linux__) || defined(__FreeBSD__) || defined(__NetBSD__) || (defined(__APPLE__) && defined(__MACH__))
 #define HAS_LUTIMES 1
 #endif
 
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <unistd.h>
 #include <stdio.h>
-#include <string.h>
+#include <unistd.h>
 #include <stdlib.h>
 #include <stddef.h>
 #include <stdarg.h>
+#include <string.h>
 #include <time.h>
 #include <errno.h>
+#include <fcntl.h>
+#include <sys/stat.h>
+#include <sys/types.h>
+#if defined(__linux__) || defined(__GLIBC__)
+#include <sys/sysmacros.h>
+#endif
 #ifdef HAS_LUTIMES
 #include <sys/time.h>
 #else
@@ -203,9 +209,11 @@ static int mkdirpath(const char *name, mode_t mode) {
 	}
 	free(buf);
 
-	if (mkdir(name, mode) < 0 &&
-	    (stat(name, &st) < 0 || !S_ISDIR(st.st_mode)))
-		return -1;
+	if (mkdir(name, mode) < 0) {
+		if (stat(name, &st) < 0 || !S_ISDIR(st.st_mode))
+			return -1;
+		chmod(name, mode);
+	}
 
 	return 0;
 }
@@ -522,7 +530,7 @@ void process_chunk(void) {
 			break;
 		case YAFFS_OBJECT_TYPE_DIRECTORY:
 			if (pt->t.objectId != YAFFS_OBJECTID_ROOT &&
-			    mkdir(obj->path_name, oh.yst_mode & STD_PERMS) < 0)
+			    mkdirpath(obj->path_name, oh.yst_mode & STD_PERMS) < 0)
 					prt_err(1, errno, "Can't create directory %s", obj->path_name);
 			safe_lchown(obj->path_name, oh.yst_uid, oh.yst_gid);
 			if ((pt->t.objectId == YAFFS_OBJECTID_ROOT ||
